#!/usr/bin/python

import cherrypy
import sys
import os
from IPython import embed
import pprint

# Tool to disable process_request_body, so we can handle file uploads
# internally.
def disable_prb():
    cherrypy.request.process_request_body = False
cherrypy.tools.disable_prb = cherrypy.Tool('before_request_body', disable_prb)

# Workaround cherrypy bug #1131 using a wrapper with fixed
# __iter__() and readline() routines
class FixChunkedRFile(object):
    def __init__(self, rfile):
        self.rfile = rfile

    def readline(self):
        data = ""
        while True:
            if not self.rfile.buffer:
                self.rfile._fetch()
                if not self.rfile.buffer:
                    return data
            newline_pos = self.rfile.buffer.find('\n')
            if newline_pos == -1:
                data += self.rfile.buffer
                self.rfile.buffer = ""
            else:
                data += self.rfile.buffer[:newline_pos+1]
                self.rfile.buffer = self.rfile.buffer[newline_pos+1:]
                return data

    def __iter__(self):
        line = self.readline()
        while line:
            yield line
            line = self.readline()

class Root(object):
    @cherrypy.expose
    @cherrypy.tools.disable_prb()
    def index(self):
        print "handling request"
        rfile = FixChunkedRFile(cherrypy.request.rfile)
        for line in rfile:
            print "got line: " + line.rstrip()

cherrypy.config.update({
    'server.socket_host': '127.0.0.1',
    'server.socket_port': 1234,
    })

cherrypy.quickstart(Root())




